;
; MC68008 prog #2
; Date started  - 16th September 86
; Date finished - 19th September 86
; Excuse - the RI stack
;
; An extension to Super Basic to get the number of free sectors on a
; medium .  Called MD_FREE

; As QDOS needs a channel ID for FS.MDINF then we need to have a
; channel open to the medium.The only file which is guaranteed to be on
; the medium is the directory.
; So we :
; (a) get and validate parameters from Super Basic
; (b) open directory file
; (c) using the channel ID,call FS.MDINF
; (d) put the number of good sectors on the RI stack
; (e) return to basic
; Step 0 - link this function with Super Basic
;
        LEA.L   PROCDEF,A1
        MOVE.W  $110,A2
        JSR     (A2)
        RTS
        
PROCDEF DC.W    0               ; No procedures
        DC.W    0               ; End of procedures
        DC.W    1               ; One function
        DC.W    START-*         ; Pointer to start of routine
        DC.B    7               ; Length of function name
        DC.B    "MD_FREE",0     ; The functions name
        DC.W    0               ; End of functions
        
; Step (a) - get and make sure of one string from Basic
;
START   MOVE.W  $116,A2
        JSR     (A2)
        BNE.S   ERR_BP
        SUBQ    #1,D3
        BNE.S   ERR_BP          ; If error then back to basic
        
; Now we have one string parameter from Super Basic
; The return parameters for CA.GTSTR are :
; (A3) = the first parameter in the name table
; (A5) = the last parameter in the name table
; the first argument is at the lowest address which is (A6,A1.L)
; D3 = the number of arguments fetched
; so CMPI.W #1,D3 is used to check there is only one string
; Note that the # and separator flags are destroyed.
; Arguments can be processed one at a time by extracting the # and
; separator flags.A5 should then be set to 8 bytes greater than A3 and
; the appropriate routine called.


      
; Step (b)  - Open a channel to the directory file
; .B (A3)   = code describing name type
; .B (A3+1) = separator info masked into name type high byte
; .W (A3+2) = pointer to the name in the name list
; .L (A3+4) = pointer to the value

; The open routine requiresJob ID,open status and a pointer to the name

        MOVEQ   #0,D1           ; Super Basics' ID is always 0
        MOVEQ   #4,D3           ; Open directory
        LEA.L   0(A6,A1.L),A0   ; Address of name
        MOVEQ   #1,D0           ; Signal IO.OPEN is the routine
        TRAP    #2              ; Do it
        TST.L   D0              ; Test for an error
        BNE.S   Exit            ; Branch if an error occurs


; So now we have a channel ID in A0 and no errors

; Step (c) - using the channel ID,call FS.MDINF
; Setting up the registers for FS.MDINF
; D3.W is the time out
; A0 is the channel ID
; A1 is the pointer to the 10 byte buffer

        MOVEQ   #-1,D3          ; Infinite timeout
        LEA.L   buffer,A1
        MOVEQ   #$45,D0         ; signal FS.MDINF
        TRAP    #3              ; Do it
        TST.L   D0
        BNE.S   CH_CLO          ; Branch if an error occurs
;
; Now we tidy up by closing the channel
;
        BSR     CH_CLO
        
; Step (d) - put the number of good sectors onto the stack
; Now the MSW of D1 contains the number of empty sectors on the
; medium.So swap the words in D1 using SWAP D1

        SWAP    D1
        MOVE.L  $58(A6),A1
        SUBQ.L  #2,A1
        MOVE.L  A1,$58(A6)
        MOVE.W  D1,0(A6,A1.L)
        MOVEQ   #3,D4
        MOVEQ   #0,D0
Exit    RTS

; General subroutines
;


CH_CLO  MOVEQ   #4,D3
        MOVEQ   #2,D0      ; Signal IO.OPEN
        TRAP    #2
        TST.L   D0
        RTS

ERR_BP  MOVEQ   #-15,D0
        RTS
buffer  DS.W    10
